from .src import (
    BaseClient,
    BodyController,
    ControllerType,
    EmotionType,
    ExpressionController,
    ExpressionType,
    FaceRecognitionClient,
    GazeDirection,
    GazeObject,
    MotionType,
    SpeechRecognitionClient,
    STTRecognitionType,
    Text2SpeechClient,
    OutputForFaceRecognition,
    OutputForSTTRecognition,
)

__version__ = "0.2.1"



import re
from dataclasses import dataclass
from typing import Literal, Optional

from dslclib.src.base import BaseClient


@dataclass
class STTRecognitionType:
    InterimResult: Literal["interimresult"] = "interimresult"
    Result: Literal["result"] = "result"


@dataclass
class OutputForSTTRecognition:
    type: Literal["interimresult", "result"]
    result: str

    def __getitem__(self, key):
        return getattr(self, key)


class SpeechRecognitionClient(BaseClient):
    """SpeechRecognitionClient

    Google Speech Recognition APIによる音声認識の結果をソケットから受け取るクライアント
    """

    def __init__(self, ip: Optional[str] = None, port: int = 8888) -> None:
        """
        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 8888
            ソケット通信を行うポート．

        Returns
        -------

        Examples
        --------
        >>> client = SpeechRecognitionClient()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 8888
        )
        >>>
        """
        super().__init__(ip, port)
        self.sock.send("start\n".encode())

    def close(self) -> None:
        self.sock.send("stop\n".encode())
        return super().close()

    def receive_line(self) -> str:
        """
        ソケットから受け取ったバイナリコードを文字列（utf-8）に変換するメソッド

        Parameters
        ----------

        Returns
        -------
        received: str
            ソケット通信によって受け取った値を文字列に変換したもの
        """
        received = str(self.sock.recv(4096).decode())
        return received

    def listen(self, interim: bool = True) -> OutputForSTTRecognition:
        """
        データがたまってからデータを出力するメソッド．

        interimをTrueにすると、発話中でもその時点の認識結果を出力する．

        interimをFalseにすると、発話終了の上での認識結果のみを出力する．

        発話中における出力例
        ('interimresult', 'あいう')

        発話終了における出力例
        ('result', 'あいうえお')

        Parameters
        ----------
        interim: bool, default = True
            Trueのときは、発話中でもその時点でたまった認識結果を出力する

        Returns
        -------
        tuple[Literal["interimresult", "result"], str]
            (状態, 認識結果)

        Examples
        --------
        >>> client = SpeechRecognitionClient()
        >>> output = client.listen()
        >>> output.result
        こんにちは
        >>> output["result"]
        こんにちは
        """
        while True:
            received = self.receive_line()
            if interim:  # 発話中でも出力するかどうか
                matching_result = re.search(r"^interimresult:([^\n]+)\n", received)
                if matching_result is not None:
                    return OutputForSTTRecognition(
                        type=STTRecognitionType.InterimResult,
                        result=matching_result.group(1),
                    )

            matching_result = re.search(r"^result:([^\n]+)\n", received)
            if matching_result is not None:
                return OutputForSTTRecognition(
                    type=STTRecognitionType.Result,
                    result=matching_result.group(1),
                )


if __name__ == "__main__":
    client = SpeechRecognitionClient()
    for _ in range(20):
        print(client.listen())

    client.close()



from dataclasses import dataclass
from typing import Literal, Optional

from dslclib.src.base import BaseClient


@dataclass
class ExpressionType:
    """Expression

    Ericaの表情を事前定義したデータクラス．
    ExpressionType()とインスタンス化する必要はない．
    """

    MouthA: Literal["mouth-a"] = "mouth-a"
    MouthI: Literal["mouth-i"] = "mouth-i"
    MouthU: Literal["mouth-u"] = "mouth-u"
    MouthE: Literal["mouth-e"] = "mouth-e"
    MouthO: Literal["mouth-o"] = "mouth-o"
    Normal: Literal["normal"] = "normal"
    FullSmile: Literal["fullsmile"] = "fullsmile"
    Smile: Literal["smile"] = "smile"
    Bad: Literal["bad"] = "bad"
    Angry: Literal["angry"] = "angry"
    EyeClose: Literal["eye-close"] = "eye-close"
    EyeOpen: Literal["eye-open"] = "eye-open"
    EyeUp: Literal["eye-up"] = "eye-up"
    EyeDown: Literal["eye-down"] = "eye-down"


"""
MouthA: str, default = 'mouth-a'
    「あ」の口
MouthI: str, default = 'mouth-i'
    「い」の口
MouthU: str, default = 'mouth-u'
    「う」の口
MouthE: str, default = 'mouth-e'
    「え」の口
MouthO: str, default = 'mouth-o'
    「お」の口
Normal: str, default = 'normal'
    平常
FullSmile: str, default = 'fullsmile'
    満面の笑顔
Smile: str, default = 'smile'
    笑顔
Bad: str, default = 'bad'
    困った顔
Angry: str, default = 'angry'
    怒った顔
EyeClose: str, default = 'eye-close'
    目を閉じる
EyeOpen: str, default = 'eye-open'
    目を開ける
EyeUp: str, default = 'eye-up'
    上を向く
EyeDown: str, default = 'eye-down'
    下を向く
"""


class ExpressionController(BaseClient):
    """ExpressionController

    Ericaの表情をコントロールするクライアント．
    ソケットにコマンドを送ることでEricaの表情を動かす．
    """

    def __init__(self, ip: Optional[str] = None, port: int = 20000) -> None:
        """
        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 20000
            ソケット通信を行うポート．

        Returns
        -------

        Examples
        --------
        >>> client = ExpressionController()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 20000
        )
        >>>
        """
        super().__init__(ip, port)

    def express(
        self,
        expression: Optional[str] = None,
        valence: float = 0,
        arousal: float = 0,
        dominance: float = 0,
        real_intension: Optional[float] = None,
    ) -> None:
        """
        Ericaの表情を支持するメソッド．
        ExpressionTypeを用いることで，簡単に指示を行うことができる．

        ExpressionTypeに事前定義されたビルトインの表情を指示する場合は，expressionに表情のidを与える．

        独自の表情を指示する場合は，valence, arousal, dominance, real_intensionを与える．

        Parameters
        ----------
        expression: str, optional
            ビルトインの表情を指示する場合に入力する．
            ExpressionTypeを用いることで，簡単にビルトインの表情を指示することができる．
        valence: float, default = 0
            ExpressionTypeにない独自の表情を指示する時に与える．
        arousal: float, default = 0
            ExpressionTypeにない独自の表情を指示する時に与える．
        dominance: float, default = 0
            ExpressionTypeにない独自の表情を指示する時に与える．
        real_intension: float, optional
            ExpressionTypeにない独自の表情を指示する時に与える．
        """
        param_list = list()

        if expression is None:
            param_list.extend(
                [
                    f"valence {valence}",
                    f"arousal {arousal}",
                    f"dominance {dominance}",
                ]
            )
            if real_intension is not None:
                param_list.append(f"realIntention {real_intension}")
        elif expression == "smile":
            param_list.extend(
                [
                    "valence 0.3",
                    "arousal 0.2",
                    "dominance 0.1",
                ]
            )
        elif expression == "normal":
            param_list.extend(
                [
                    "valence 0",
                    "arousal 0",
                    "dominance 0",
                    "realIntention 0",
                ]
            )

        if len(param_list) > 0:
            param_list.append("expression MoodBasedFACS")
            command = "\n".join(param_list) + "\n"
            self.sock.send(command.encode())
            return
        command = f"expression {expression}\n"
        self.sock.send(command.encode())


if __name__ == "__main__":
    import time

    controller = ExpressionController()
    controller.express(ExpressionType.FullSmile)
    time.sleep(3)
    controller.express(ExpressionType.Smile)
    time.sleep(3)
    controller.express(valence=0.2, arousal=0.8, dominance=0.4)
    controller.close()



import socket
from typing import Optional


class BaseClient(object):
    """BaseClient"""

    def __init__(self, ip: Optional[str] = None, port: int = 8888) -> None:
        """
        ソケット通信をする各モジュールのベースクラス

        各モジュールはこのBaseClientを継承する

        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 8888
            ソケット通信を行うポート．

            BaseClientを継承するときには，そのクラスに合わせてportのデフォルトを書き換える．

        Returns
        -------

        Examples
        --------
        >>> client = BaseClient()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 8888
        )
        >>>
        >>> client = BaseClient(ip='localhost', port=3333)
        >>> client
        Socket(
            ip   = localhost
            port = 3333
        )
        """
        self.sock: socket.socket = socket.socket()
        self.ip: str = "127.0.0.1"
        if ip is not None:
            self.ip = ip
        else:
            try:
                with open("/etc/hosts", "r") as f:
                    for line in f.readlines():
                        if "host.docker.internal" in line:
                            self.ip = (
                                "host.docker.internal"  # dockerにホストのポートを接続許可した時に上書き
                            )
                            break
            except Exception:  # /etc/hostsが存在しないとき
                pass
            finally:
                print(f"ipがNoneだったため、{self.ip}をipアドレスとして設定します。")

        self.port: int = port

        self.connect()

    def __repr__(self) -> str:
        return f"""Socket(
            ip   = {self.ip}
            port = {self.port}
        )"""

    def connect(self) -> None:
        """ipアドレスのportに接続する．

        Parameters
        ----------

        Returns
        -------

        Examples
        --------
        >>> client = BaseClient()
        >>> client.connect()

        """
        self.sock.connect((self.ip, self.port))

    def close(self) -> None:
        """ipアドレスのportへの接続を切る．

        Parameters
        ----------

        Returns
        -------

        Examples
        --------
        >>> client = BaseClient()
        >>> client.connect()
        >>> client.close()

        """
        self.sock.close()



import json
import re
from collections import Counter
from dataclasses import dataclass
from typing import Callable, Literal, Optional

from dslclib.src.base import BaseClient


@dataclass
class Rotation:
    """Rotation
    ユーザの顔の向きを示すデータクラス

    各アトリビュートは以下の通りです．

    - pitch: 上下(上がプラス)
    - roll: 回転．首をかしげる動き(右にかしげるとプラス)
    - yaw: 横向き(左がプラス)
    """

    pitch: float
    roll: float
    yaw: float


@dataclass
class OutputForFaceRecognition:
    """OutputForFaceRecognition
    感情認識サーバのクライアントが出力するデータクラス

    各アトリビュートは以下の通りです．

    - summarized: 出力結果が，funcによって，一定時刻分の認識結果を要約されたものであるかどうか
    - timestamp: タイムスタンプ
    - emotion: 認識された感情クラス
    - emotion_score: 感情クラスのスコア
    - rotations: 各時刻のRotationの情報が時系列順に含まれるリスト
    - age: 推測された年齢
    - gender: 推測された性別
    - gender_score: 推測性別のスコア
    """

    summarized: bool
    timestamp: float
    emotion: str
    emotion_score: float
    rotations: list[Rotation]
    age: Optional[int] = None
    gender: Optional[Literal["Man", "Woman"]] = None
    gender_score: Optional[float] = None

    def __getitem__(self, key):
        return getattr(self, key)


@dataclass
class EmotionType:
    """EmotionType
    感情認識サーバが出力する感情値を定義したデータクラス
    """

    Neutral: Literal["neutral"] = "neutral"
    Happiness: Literal["happiness"] = "happiness"
    Surprise: Literal["surprise"] = "surprise"
    Sadness: Literal["sadness"] = "sadness"
    Anger: Literal["anger"] = "anger"
    Disgust: Literal["disgust"] = "disgust"
    Fear: Literal["fear"] = "fear"


class FaceRecognitionClient(BaseClient):
    """FaceRecognitionClient

    ユーザの感情を認識するサーバとソケット通信するクライアント．
    （これから顔の向きに対応する予定）
    """

    def __init__(self, ip: Optional[str] = None, port: int = 4500) -> None:
        """
        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 4500
            ソケット通信を行うポート．

        Returns
        -------

        Examples
        --------
        >>> client = FaceRecognitionClient()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 4500
        )
        >>>
        """
        super().__init__(ip, port)
        self.age: Optional[int] = None
        self.gender: Optional[Literal["Man", "Woman"]] = None
        self.gender_score: Optional[float] = None

    def get_user_info(
        self,
    ) -> tuple[Optional[int], Optional[Literal["Man", "Woman"]], Optional[float]]:
        return self.age, self.gender, self.gender_score

    def receive_line(self) -> str:
        """
        ソケットから受け取ったバイナリコードを文字列（utf-8）に変換するメソッド

        Parameters
        ----------

        Returns
        -------
        received: str
            ソケット通信によって受け取った値を文字列に変換したもの
        """
        received = str(self.sock.recv(256).decode())
        return received

    @staticmethod
    def summarize_times(
        data: list[tuple[float, str, float]],
        times: int = 10,
        strategy: Literal["majority", "latest"] = "majority",
    ) -> tuple[bool, Optional[float], Optional[str], Optional[float]]:
        """
        感情認識サーバから得たtimes回の結果を集約し，strategyに則ったアルゴリズムで感情値を決定する静的メソッド．

        strategyはmajorityを推奨する．

        Parameters
        ----------
        data: list[tuple[float, str, float]]
            各時刻における感情認識結果がプールされたリスト
        times: int, default = 10
            何回分の結果をプールしたら要約するか
        strategy: str, default = 'majority'
            majorityならば多数決、latestならば最新の認識結果を認識結果とする

        Returns
        -------
        tuple[bool, Optional[float], Optional[str], Optional[float]]
            (戻り値があるか？, timestamp, 感情値, 感情スコア)

            1つ目の要素がTrueならば，指定回数分の結果を用いて感情値を決定していることを示す．
            Falseならば，回数分に満たないため，さらにプールを要求することを示す．
        """
        if len(data) <= times:
            return False, None, None, None

        timestamps, emotions, scores_ = zip(*data)
        scores = list(scores_)
        if strategy == "majority":
            counter = Counter(emotions)
            emotion = counter.most_common(1)[0][0]
            timestamp = timestamps[-1]
            scores = [s for e, s in zip(emotions, scores) if e == emotion]
            score = sum(scores) / len(scores)
        elif strategy == "latest":
            emotion = emotions[-1]
            timestamp = timestamps[-1]
            score = scores[-1]
        else:
            raise ValueError("strategyの引数には、`majority`と`latest`のみが有効です")

        return True, timestamp, emotion, score

    @staticmethod
    def summarize_timestamps(
        data: list[tuple[float, str, float]],
        sec: float = 1,
        strategy: Literal["majority", "latest"] = "majority",
    ) -> tuple[bool, Optional[float], Optional[str], Optional[float]]:
        """
        感情認識サーバから得たsec秒分の結果を集約し，strategyに則ったアルゴリズムで感情値を決定する静的メソッド．

        strategyはmajorityを推奨する．

        Parameters
        ----------
        data: list[tuple[float, str, float]]
            各時刻における感情認識結果がプールされたリスト
        sec: float, default = 1
            何秒プールしたら認識結果を要約するか
        strategy: str, default = 'majority'
            majorityならば多数決、latestならば最新の認識結果を認識結果とする

        Returns
        -------
        tuple[bool, Optional[float], Optional[str], Optional[float]]
            1つ目の要素がTrueならば，指定回数分の結果を用いて感情値を決定していることを示す．
            Falseならば，回数分に満たないため，さらにプールを要求することを示す．
        """
        if data[-1][0] - data[0][0] < sec:
            return False, None, None, None

        timestamps, emotions, scores_ = zip(*data)
        scores = list(scores_)
        if strategy == "majority":
            counter = Counter(emotions)
            emotion = counter.most_common(1)[0][0]
            timestamp = timestamps[-1]
            scores = [s for e, s in zip(emotions, scores) if e == emotion]
            score = sum(scores) / len(scores)
        elif strategy == "latest":
            emotion = emotions[-1]
            timestamp = timestamps[-1]
            score = scores[-1]
        else:
            raise ValueError("strategyの引数には、`majority`と`latest`のみが有効です")

        return True, timestamp, emotion, score

    def listen(self, func: Optional[Callable] = None) -> OutputForFaceRecognition:
        """
        感情認識サーバとのソケット通信をするメソッド

        Parameters
        ----------
        func: Callable, optional
            funcに何も与えない場合、感情認識結果が取得された時点ですぐに結果を返します.

            ただし、認識結果の応答が早く、安定しないこともあることから、funcを与えないことは推奨しません.

            カスタムでfuncを定義してもよいですが、簡易的な静的メソッドとして`summarize_times`と`summarize_timestamps`を用意してあります.
            カスタムで定義する場合の要件は以下です.
            入力: list[tuple[float, str, float]]

            リストの各要素は各時点での認識結果であり、
            認識結果は (timestamp, emotion_class, emotion_score)です.

            出力: tuple[bool, Optional[float], Optional[str], Optional[float]]

            (要約した認識結果を返す=True | まだプールする=False, timestamp, emotion_class, emotion_score)
            以上の要件を満たした関数を設計してください.

        Returns
        -------
        OutputForFaceRecognition

        Examples
        --------
        >>> client = FaceRecognitionClient()
        >>> output = client.listen(FaceRecognitionClient.summarize_times)
        >>> output.emotion
        angry
        >>> output["emotion"]
        angry
        """
        do_return: bool = False
        pool: list[tuple[float, str, float]] = []
        rotations: list[Rotation] = []
        while True:
            received = self.receive_line()
            if not re.search(r"^\{[^\n]+\}\n", received):
                continue

            data = json.loads(received)
            timestamp: float = data["timestamp"]
            emotion: str = data["emotion_class"]
            score: float = data["emotion_score"]
            rot = data["rotation"]
            rotations.append(
                Rotation(pitch=rot["pitch"], roll=rot["roll"], yaw=rot["yaw"])
            )
            # user info
            if "age" in list(data.keys()):
                self.age = data["age"]
                self.gender = data["gender_class"]
                self.gender_score = data["gender_score"]

            age, gender, gender_score = self.get_user_info()
            if func is None:
                return OutputForFaceRecognition(
                    summarized=True,
                    timestamp=timestamp,
                    emotion=emotion,
                    emotion_score=score,
                    rotations=rotations,
                    age=age,
                    gender=gender,
                    gender_score=gender_score,
                )

            pool.append((timestamp, emotion, score))
            do_return, timestamp, emotion, score = func(data=pool)
            if do_return:
                if not isinstance(emotion, str):
                    raise TypeError(
                        f"出力する感情クラスは`str`のみですが、{type(str)}があたえられています。`func`を要件を満たすように設計してください。"
                    )
                return OutputForFaceRecognition(
                    summarized=True,
                    timestamp=timestamp,
                    emotion=emotion,
                    emotion_score=score,
                    rotations=rotations,
                    age=age,
                    gender=gender,
                    gender_score=gender_score,
                )


if __name__ == "__main__":
    client = FaceRecognitionClient()
    for i in range(10):
        output = client.listen(FaceRecognitionClient.summarize_times)
        print(output)
        print(output.emotion)
        print(output["emotion"])

    for _ in range(3):
        print(client.listen(FaceRecognitionClient.summarize_timestamps))

    client.close()



from .base import BaseClient
from .speech_recognition import SpeechRecognitionClient, STTRecognitionType, OutputForSTTRecognition
from .tts import Text2SpeechClient
from .expression_controller import ExpressionController, ExpressionType
from .body_controller import MotionType, GazeDirection, GazeObject, ControllerType, BodyController
from .face_recognition import EmotionType, FaceRecognitionClient, OutputForFaceRecognition



import json
from dataclasses import dataclass
from typing import Literal, Optional

from dslclib.src.base import BaseClient


@dataclass
class MotionType:
    """MotinonType
    Ericaのモーションの命令を一部事前定義している．
    MotionType()とインスタンス化する必要はない．

    ここでは，Default, Greeting, Nono, NodDeep, Nod, RightHandBasePosition, LeftHandBasePositionを用意している．

    以下のプロパティ以外は、Erica@CGのMotionを参照されたい．
    CGEricaの画面の左上のウィンドウに全てのモーションの命令が書かれている．

    Examples
    --------
    >>> MotionType.Default
    DefaultMotion
    >>> MotionType.Nod
    nod
    >>> MotionType.RightHandBasePosition
    righthandbaseposition
    """

    Default: Literal["DefaultMotion"] = "DefaultMotion"
    Greeting: Literal["greeting"] = "greeting"
    Nono: Literal["nono"] = "nono"
    NodDeep: Literal["nod_deep"] = "nod_deep"
    Nod: Literal["nod"] = "nod"
    RightHandBasePosition: Literal["righthandbaseposition"] = "righthandbaseposition"
    LeftHandBasePositin: Literal["lefthandbaseposition"] = "lefthandbaseposition"


"""
Default: str, default = "DefaultMotion"
    デフォルトの姿勢にする
Greeting: str, default = "greeting"
    挨拶をする．軽い会釈をする．
Nono: str, default = "nono"
    ノーノー．首を振る．
NodDeep: str, default = "nod_deep"
    深く頷く
Nod: str, default = "nod"
    頷く
RightHandBasePosition: str, default = "righthandbaseposition"
    右手を基準の場所に戻す
LeftHandBasePosition: str, default = "lefthandbaseposition"
    左手を基準の場所に戻す
"""


@dataclass
class GazeDirection:
    """GazeDirection
    Ericaの目線の方向を事前定義したデータクラス．
    GazeDirection()とインスタンス化する必要はない．

    FrontとRightとLeftを用意している．
    この3方向以外の向きを指定する場合，BodyController.gazeに直接数値を入力することができる．

    Examples
    --------
    >>> GazeDirection.Front
    (0.0, 1.2, 1.5)
    >>> GazeDirection.Right
    (1.0, 1.2, 1.5)
    >>> GazeDirection.Left
    (-1.0, 1.2, 1.5)
    """

    Front: tuple[float, float, float] = (0.0, 1.2, 1.5)
    Right: tuple[float, float, float] = (1.0, 1.2, 1.5)
    Left: tuple[float, float, float] = (-1.0, 1.2, 1.5)


@dataclass
class GazeObject:
    """GazeObject

    Ericaの目線の対象を事前定義したデータクラス．
    GazeObject()とインスタンス化する必要はない．

    Desk，Monitor，Sofaを用意している．

    Examples
    --------
    >>> GazeObject.Monitor
    monitor
    >>> GazeObject.Desk
    desk
    >>> GazeObject.Sofa
    Sofa
    """

    Monitor: Literal["monitor"] = "monitor"
    Desk: Literal["desk"] = "desk"
    Sofa: Literal["Sofa"] = "Sofa"


"""
Monitor: str, default = "monitor"
    Ericaの目線をモニターに向ける
Desk: str, default = "desk"
    Ericaの目線をデスクに向ける
Sofa: str, default = "Sofa"
    Ericaの目線をソファに向ける
"""


@dataclass
class ControllerType:
    """ControllerType

    Ericaのどの部分をコントロールするかを指定するデータクラス．
    ControllerType()とインスタンス化する必要はない．

    EyeControllerはEricaの目線のコントロールを指定し，HeadControllerはEricaの頭の向きのコントロールを指定する．
    指定する文字列が長いため，このデータクラスを使うことを推奨する．

    Examples
    --------
    >>> ControllerType.Eye
    EyeController
    >>> ControllerType.Head
    HeadController
    """

    Eye: Literal["EyeController"] = "EyeController"
    Head: Literal["HeadController"] = "HeadController"


"""
Eye: str, default = "EyeController"
    目線を動かすことを指示
Head: str, default = "HeadController"
    顔の向きを動かすことを指示
"""


class BodyController(BaseClient):
    """BodyController

    Ericaの身体をコントロールするクライアント．
    ソケットにコマンドを送ることでEricaを動かす．
    """

    def __init__(self, ip: Optional[str] = None, port: int = 21000) -> None:
        """
        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 21000
            ソケット通信を行うポート．

        Returns
        -------

        Examples
        --------
        >>> client = BodyController()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 21000
        )
        >>>
        """
        super().__init__(ip, port)

    def play_motion(self, motion: str) -> None:
        """
        モーション名に対応するモーションをEricaに指示するメソッド

        Parameters
        ----------
        motion: str
            モーション名．MotionTypeのデータクラスを使うことで，簡単に指示を行うことができる．
            MotionTypeでサポートしていないモーション名もたくさんあり，これらはCGEricaの画面左上で確認することができる．
            より詳細なモーションを指示したい場合は，直接strを引数に与えること．

        Returns
        -------

        Examples
        --------
        >>> client.play_motion(MotionType.Greeting)
        >>> client.play_motion(motion=MotionType.Greeting)
        >>> client.play_motion("greeting")
        >>>
        """
        command = f"playmotion={motion}\n"
        self.sock.send(command.encode())

    def gaze(
        self,
        obj: Optional[str] = None,
        direction: Optional[tuple[float, float, float]] = None,
        eye: Optional[tuple[float, float, float]] = None,
        head: Optional[tuple[float, float, float]] = None,
    ) -> None:
        """
        Ericaの目線または顔の向きを指示するメソッド．
        GazeObject，GazeDirectionを用いることで，簡単に指示を行うことができる．

        obj, direction, eye, headのいずれかは引数として入力しなければならない．

        目線と顔の向きの両方を，同じ向きに動かす場合はdirectionに座標を与える．

        目線のみならばeye，顔の向きのみならばheadに座標を与える．

        また，目線と顔の向きを動かすが，別の座標をそれぞれ指示したい場合は，eyeとheadにそれぞれ座標を与える．

        Parameters
        ----------
        obj: str, optional
            見るオブジェクトの名前．指定したオブジェクトの方向に目線と顔を向ける．
            ここではデータクラスGazeObjectを用いることができる．
            直接文字列を与えることも可能．
        direction: tuple, optional
            指定した座標に対して目線と顔を向ける．
            ここではデータクラスGazeDirectionを用いることができる．
            直接座標をタプルで与えることも可能．
        eye: tuple, optional
            指定した座標に対して目線のみを向ける．
            ここではデータクラスGazeDirectionを用いることができる．
            直接座標をタプルで与えることも可能．
        head: tuple, optional
            指定した座標に対して顔のみを向ける．
            ここではデータクラスGazeDirectionを用いることができる．
            直接座標をタプルで与えることも可能．

        Returns
        -------

        Examples
        --------
        >>> client.gaze(obj=GazeObject.Sofa)
        >>> client.gaze(direction=GazeDirection.Right)  # 目線と顔の向きを右に
        >>> client.gaze(eye=GazeDirection.Right)  # 目線のみを右に
        >>> client.gaze(eye=GazeDirection.Right, head=(0.0, 1.0, 2.0))  # 目線と顔の向きを別にできる．また，座標を直接与えられる．
        >>> client.gaze()  # 何も引数を与えないとエラー
        ValueError("いずれかの引数に正しい引数を入力してください。")
        """
        if obj is None and direction is None and eye is None and head is None:
            raise ValueError("いずれかの引数に正しい引数を入力してください。")

        if obj is not None:
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Eye,
                    obj=obj,
                    direction=(0, 0, 0),
                ).encode()
            )
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Head,
                    obj=obj,
                    direction=(0, 0, 0),
                ).encode()
            )

        if direction is not None:
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Eye,
                    obj="",
                    direction=direction,
                ).encode()
            )
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Head,
                    obj="",
                    direction=direction,
                ).encode()
            )

        if eye is not None:
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Eye,
                    obj="",
                    direction=eye,
                ).encode()
            )

        if head is not None:
            self.sock.send(
                self._create_command(
                    controller=ControllerType.Head,
                    obj="",
                    direction=head,
                ).encode()
            )

    def _create_command(
        self,
        controller: str,
        obj: str,
        direction: tuple[float, float, float],
    ) -> str:
        cmd = {
            "id": controller,
            "motionTowardObject": obj,
            "targetMotionMode": 2,
            "targetPoint": {"x": direction[0], "y": direction[1], "z": direction[2]},
            "translateSpeed": 2.0,
        }
        return f"{controller}={json.dumps(cmd)}\n"


if __name__ == "__main__":
    import time

    controller = BodyController()

    controller.play_motion(MotionType.Greeting)
    time.sleep(2)
    controller.play_motion(MotionType.Nono)
    time.sleep(2)

    controller.gaze(GazeObject.Monitor)
    time.sleep(3)
    controller.gaze(direction=GazeDirection.Left)
    controller.gaze(direction=GazeDirection.Front)
    time.sleep(3)

    controller.gaze(eye=GazeDirection.Front, head=GazeDirection.Right)

    controller.close()



import json
from typing import Optional

from dslclib.src.base import BaseClient


class SpeechConfig(str):
    """
    SpeechConfig

    スピーチの設定の入力から，文字列を生成するstr継承クラス
    """

    def __new__(
        cls,
        text: str,
        engine: str = "POLLY",
        speaker: str = "Mizuki",
        pitch: int = 100,
        volume: int = 100,
        speed: int = 100,
        vocal_tract_length: int = 0,
        duration_information: bool = False,
        speechmark: bool = False,
    ):
        """SpeechConfig

        Parameters
        ----------
        text: str
        engine: str, default = 'POLLY'
        speaker: str, default = 'Mizuki'
        pitch: int, default = 100
        volume: int, default = 100
        speed: int, default = 100
        vocal_tract_length: int, default = 0
        duration_information: bool, default = False
        speechmark: bool, default = False
        """
        data = {
            "engine": engine,
            "speaker": speaker,
            "pitch": pitch,
            "volume": volume,
            "speed": speed,
            "vocal-tract-length": vocal_tract_length,
            "duration-information": duration_information,
            "speechmark": speechmark,
            "text": text,
        }
        return super().__new__(cls, json.dumps(data, ensure_ascii=False))


class Text2SpeechClient(BaseClient):
    """Text2SpeechClient

    音声にしたい発話テキストをソケットに送信するクライアント
    """

    def __init__(self, ip: Optional[str] = None, port: int = 3456) -> None:
        """
        Parameters
        ----------
        ip: str, optional
            ipアドレス．

            デフォルトはNoneであり，Noneが与えられた時，127.0.0.1(ローカルホスト)を指定し，
            もし，docker内でこのモジュールが立ち上がっていた場合，自動でそれが認識され，host.docker.internalを指定する．

            host.docker.internalは，docker内からローカルホストのポートに接続するために必要である．
        port: int = 3456
            ソケット通信を行うポート．

        Returns
        -------

        Examples
        --------
        >>> client = Text2SpeechClient()
        ipがNoneだったため、127.0.0.1をipアドレスとして設定します。
        >>> client
        Socket(
            ip   = 127.0.0.1
            port = 3456
        )
        >>>
        """
        super().__init__(ip, port)

    def speech(
        self,
        text: str,
        engine: str = "POLLY",
        speaker: str = "Mizuki",
        pitch: int = 100,
        volume: int = 100,
        speed: int = 100,
        vocal_tract_length: int = 0,
        duration_information: bool = False,
        speechmark: bool = False,
    ) -> None:
        """
        音声にしたい発話テキストと発話音声の設定を入力としてソケットに送信する関数

        Parameters
        ----------
        text: str
            発話テキスト
        engine: str, default = 'POLLY'
            音声エンジン
        speaker: str, default = 'Mizuki'
            話者
        pitch: int, default = 100
            ピッチ
        volume: int, default = 100
            ボリューム
        speed: int, default = 100
            発話スピード
        vocal_tract_length: int, default = 0
        duration_information: bool, default = False
        speechmark: bool, default = False
            スピーチマークが含まれるかどうか


        Returns
        -------
        """
        speech = SpeechConfig(
            text=text,
            engine=engine,
            speaker=speaker,
            pitch=pitch,
            volume=volume,
            speed=speed,
            vocal_tract_length=vocal_tract_length,
            duration_information=duration_information,
            speechmark=speechmark,
        )
        command = speech + "\n"
        self.sock.send(command.encode())


if __name__ == "__main__":
    import time

    client = Text2SpeechClient()
    client.speech("あーーーー", speed=50)
    time.sleep(3)
    client.speech("すみません", volume=20)
    time.sleep(3)
    client.speech("こんにちは")

    client.close()
